/*****************************************************************************/
/* Analyzer.cpp  Analyzer engine        (C)Copyright Gregory J. Shrago 1997  */
/*****************************************************************************/
#include "analyzer.h"

// operators in priority order
const char* OPERATORS = "n+-*/^f";
// functions - no order
const char*  FUNCTIONS[] = {"",
                            "sin",   "cos",   "tg",    "arcsin", "arccos",
                            "arctg", "ln",    "lg",    "exp",    "abs"};

int _matherr(exception *x){
  throw XMathException("Math exception");
}

int XStringAnalyzer::SIMPLIFY=1;
/*****************************************************************************/
char* XStringAnalyzer::SetupString(char* String){
int i = strlen(String)-1;
  while(String[i]=='0' && i>-1)
       String[i--] = 0x0;
  if(String[i]=='.') String[i]=0x0;
return String;
}

double XStringAnalyzer::Approximate(double v){
  if(ceil(v)-v<CONST_EPS) v=ceil(v);
  else if(v-floor(v)<CONST_EPS) v=floor(v);
  return v;
}

/*****************************************************************************/
// String constructor
XStringAnalyzer::XStringAnalyzer(char* s, int c){
char* String;
  Type = TYPE_UNKNOWN;
  Left = Right = NULL;
  Param = 0;
  String = strdup(s);
  DelParen(String).Analyze(String).SSimplify();
  if(c)
   if(!isConst()){
     char Buf[64], Temp[64];
     strcpy(Buf, "");
     try { toString(Temp,32); }
     catch(XMathException e) {}
     sprintf(Buf, "%s must be const", Temp);
     throw XMathException(Buf, 1);
   }
  free(String);
}

// Constructs object from L,R, Type and Param
XStringAnalyzer::XStringAnalyzer(int T, XStringAnalyzer* L, XStringAnalyzer* R, double P){
  Left = L; Right = R;
  Type = T; Param = P;
}

//Clones s
XStringAnalyzer::XStringAnalyzer(XStringAnalyzer* s){
  Clone(s);
}

// Frees memory buffers
XStringAnalyzer::~XStringAnalyzer(){
  if(Type == TYPE_OPERATOR){
    delete Left;
    delete Right;
  }
  if(Type == TYPE_FUNCTION)
    delete Left;
}


XStringAnalyzer& XStringAnalyzer::Clone(XStringAnalyzer *s){
  if(s==NULL){
    Type = TYPE_UNKNOWN;
    throw XMathException("Exception: cloning null object");
  }
  Param = s->Param;
  Type=s->Type;
  if(Type == TYPE_OPERATOR){
    Left  = new XStringAnalyzer(s->Left);
    Right = new XStringAnalyzer(s->Right);
  }
  if(Type == TYPE_FUNCTION){
    Left = new XStringAnalyzer(s->Left);
  }
  return *this;
}

// If necessary converts back into string
char* XStringAnalyzer::toString(char* s, int size){
char buf[64];
  if(size - strlen(s) < 10){
       strcat(s, "...");
       throw XMathException("Buffer is too small");
       return s;
  }
  switch(Type){
    case TYPE_VARIABLE:
         sprintf(buf, "x");
         strcat(s,buf);
         break;
    case TYPE_CONST:
         sprintf(buf, "%.6f", Param);
         SetupString(buf);
         if(Param<0)
         strcat(s,"(");
         strcat(s,buf);
         if(Param<0)
         strcat(s,")");
         break;
    case TYPE_OPERATOR:
         sprintf(buf, "%c", OPERATORS[(int)Param]);
         switch((int)Param){
           case OP_PLUS:
                Left->toString(s, size);
                strcat(s,buf);
                Right->toString(s, size);
                break;
           case OP_MINES:
                if(Left->Type==TYPE_CONST && Left->Param==0)
                  strcat(s,"(");
                else Left->toString(s, size);
                  strcat(s,buf);
                if(Right->Type==TYPE_OPERATOR && Right->Param < OP_MULTPLY)
                  strcat(s,"(");
                Right->toString(s, size);
                if(Right->Type==TYPE_OPERATOR && Right->Param < OP_MULTPLY)
                  strcat(s,")");
                if(Left->Type==TYPE_CONST && Left->Param==0)
                  strcat(s,")");
                break;
           case OP_MULTPLY:
                if(Left->Type==TYPE_OPERATOR && (int)Left->Param < OP_MULTPLY){
                  strcat(s,"(");
                  Left->toString(s, size);
                  strcat(s,")");
                } else Left->toString(s, size);
                strcat(s,buf);
                if(Right->Type==TYPE_OPERATOR && Right->Param < OP_MULTPLY){
                  strcat(s,"(");
                  Right->toString(s, size);
                  strcat(s,")");
                } else Right->toString(s, size);
                break;
           case OP_DIVIDE:
                if(Left->Type==TYPE_OPERATOR && Left->Param < OP_MULTPLY){
                  strcat(s,"(");
                  Left->toString(s, size);
                  strcat(s,")");
                } else Left->toString(s, size);
                strcat(s,buf);
                if(Right->Type==TYPE_OPERATOR && Right->Param <= OP_MULTPLY){
                  strcat(s,"(");
                  Right->toString(s, size);
                  strcat(s,")");
                } else Right->toString(s, size);
                break;
           case OP_POWER:
                if(Left->Type==TYPE_OPERATOR)
                  strcat(s,"(");
                Left->toString(s, size);
                if(Left->Type==TYPE_OPERATOR)
                  strcat(s,")");
                strcat(s,buf);
                if(Right->Type==TYPE_OPERATOR)
                  strcat(s,"(");
                Right->toString(s, size);
                if(Right->Type==TYPE_OPERATOR)
                  strcat(s,")");
                break;
           default: throw XMathException("Exception: unknown operator", 1);
         }
         break;
    case TYPE_FUNCTION:
         strcat(s, FUNCTIONS[(int)Param]);
         strcat(s,"(");
         Left->toString(s, size);
         strcat(s,")");
         break;
    default: throw XMathException("Exception: unknown type", 1);
  }
return s;
}

/*****************************************************************************/
// Calculates F(x) at the specified x
double XStringAnalyzer::Count(double x){
double R, L;
double rc;
 switch(Type){
   case TYPE_VARIABLE:
        rc = x;
        break;
   case TYPE_CONST:
        rc = Param;
        break;
   case TYPE_OPERATOR:
        L=Left->Count(x);
        R=Right->Count(x);
        switch((int)Param){
          case OP_PLUS:    rc = L+R; break;
          case OP_MINES:   rc = L-R; break;
          case OP_MULTPLY: rc = L*R; break;
          case OP_DIVIDE:  if(!R) throw XMathException("division by zero");
                           rc = L/R; break;
          case OP_POWER:   if(L<0  && floor(R)!=R) throw XMathException("incorrect powering");
                           if(L==0 && R < 0) throw XMathException("incorrect powering");
                           rc = pow(L,R); break;
          default: throw XMathException("Exception: unknown operator", 1);
        }
        break;
   case TYPE_FUNCTION:
        L=Left->Count(x);
        switch((int)Param){
          case FUNC_SIN:   rc = sin(L);   break;
          case FUNC_COS:   rc = cos(L);   break;
          case FUNC_TG:    if(fabs(2*L/CONST_PI)==1)  throw XMathException("tg(x): x can\'t be k*pi/2");
                           rc = tan(L);   break;
          case FUNC_ASIN:  if(fabs(L)>1)  throw XMathException("arcsin(x): x must be in [-1;1]");
                           rc = asin(L);  break;
          case FUNC_ACOS:  if(fabs(L)>1)  throw XMathException("arccos(x): x must be in [-1;1]");
                           rc = acos(L);  break;
          case FUNC_ATG:   rc = atan(L);  break;
          case FUNC_LN:    if(L<=0)  throw XMathException("ln(x): x must be in (0;+~)");
                           rc = log(L);   break;
          case FUNC_LG:    if(L<=0)  throw XMathException("lg(x): x must be in (0;+~)");
                           rc = log10(L); break;
          case FUNC_EXP:   rc = exp(L);   break;
          case FUNC_ABS:   rc = fabs(L);  break;
          default: throw XMathException("Exception: unknown function", 1);
        }
        break;
   default: throw XMathException("Exception: unknown type", 1);
 }
 return rc;
}

// Calculates derivative function
XStringAnalyzer* XStringAnalyzer::Derivative()
{
XStringAnalyzer* D=NULL;
XStringAnalyzer *L = NULL, *R = NULL;
 switch(Type){
   case TYPE_VARIABLE:
        D=new XStringAnalyzer(TYPE_CONST, NULL, NULL, 1);
        break;
   case TYPE_CONST:
        D=new XStringAnalyzer(TYPE_CONST, NULL, NULL, 0);
        break;
   case TYPE_OPERATOR:
        L=Left->Derivative();
        R=Right->Derivative();
        switch((int)Param){
          case OP_PLUS:
               D = new XStringAnalyzer(TYPE_OPERATOR, L, R, OP_PLUS);
               break;
          case OP_MINES:
               D = new XStringAnalyzer(TYPE_OPERATOR, L, R, OP_MINES);
               break;
          case OP_MULTPLY:
               D = new XStringAnalyzer(TYPE_OPERATOR,
                    new XStringAnalyzer(TYPE_OPERATOR,
                     L, new XStringAnalyzer(Right), OP_MULTPLY),
                    new XStringAnalyzer(TYPE_OPERATOR,
                     new XStringAnalyzer(Left), R, OP_MULTPLY),
                    OP_PLUS);
               break;
          case OP_DIVIDE:
               D = new XStringAnalyzer(TYPE_OPERATOR,
                    new XStringAnalyzer(TYPE_OPERATOR,
                     new XStringAnalyzer(TYPE_OPERATOR,
                      L, new XStringAnalyzer(Right), OP_MULTPLY),
                     new XStringAnalyzer(TYPE_OPERATOR,
                      new XStringAnalyzer(Left), R, OP_MULTPLY),
                      OP_MINES),
                    new XStringAnalyzer(TYPE_OPERATOR,
                     new XStringAnalyzer(Right),
                     new XStringAnalyzer(TYPE_CONST, NULL, NULL, 2), OP_POWER),
                    OP_DIVIDE);
               break;
          case OP_POWER:
               D = new XStringAnalyzer(TYPE_OPERATOR,
                    new XStringAnalyzer(this),
                    new XStringAnalyzer(TYPE_OPERATOR,
                     new XStringAnalyzer(TYPE_OPERATOR,
                      R,
                      new XStringAnalyzer(TYPE_FUNCTION,
                       new XStringAnalyzer(Left), NULL, FUNC_LN),
                      OP_MULTPLY),
                     new XStringAnalyzer(TYPE_OPERATOR,
                      new XStringAnalyzer(Right),
                      new XStringAnalyzer(TYPE_OPERATOR,
                       L, new XStringAnalyzer(Left), OP_DIVIDE),
                      OP_MULTPLY),
                     OP_PLUS),
                    OP_MULTPLY);
               break;
          default: throw XMathException("Exception: unknown function", 1);
        }
        break;
   case TYPE_FUNCTION:
        L=Left->Derivative();
        switch((int)Param){
          case FUNC_SIN:  R = new XStringAnalyzer(TYPE_FUNCTION, new XStringAnalyzer(Left), NULL, FUNC_COS);
                          break;
          case FUNC_COS:  R = new XStringAnalyzer(TYPE_OPERATOR,
                               new XStringAnalyzer(TYPE_CONST, NULL, NULL, 0),
                               new XStringAnalyzer(TYPE_FUNCTION,
                                new XStringAnalyzer(Left), NULL, FUNC_SIN),
                               OP_MINES);
                          break;
          case FUNC_TG:   R = new XStringAnalyzer(TYPE_OPERATOR,
                               new XStringAnalyzer(TYPE_FUNCTION,
                                new XStringAnalyzer(Left), NULL, FUNC_COS),
                               new XStringAnalyzer(TYPE_CONST, NULL, NULL, -2),
                               OP_POWER);
                          break;
          case FUNC_ASIN: R = new XStringAnalyzer(TYPE_OPERATOR,
                               new XStringAnalyzer(TYPE_OPERATOR,
                                new XStringAnalyzer(TYPE_CONST, NULL, NULL, 1),
                                new XStringAnalyzer(TYPE_OPERATOR,
                                 new XStringAnalyzer(Left),
                                 new XStringAnalyzer(TYPE_CONST, NULL, NULL, 2),
                                 OP_POWER),
                                OP_MINES),
                               new XStringAnalyzer(TYPE_CONST, NULL, NULL, -.5),
                               OP_POWER);
                          break;
          case FUNC_ACOS: R = new XStringAnalyzer(TYPE_OPERATOR,
                               new XStringAnalyzer(TYPE_CONST, NULL, NULL, 0),
                               new XStringAnalyzer(TYPE_OPERATOR,
                                new XStringAnalyzer(TYPE_OPERATOR,
                                 new XStringAnalyzer(TYPE_CONST, NULL, NULL, 1),
                                 new XStringAnalyzer(TYPE_OPERATOR,
                                  new XStringAnalyzer(Left),
                                  new XStringAnalyzer(TYPE_CONST, NULL, NULL, 2),
                                  OP_POWER),
                                 OP_MINES),
                                new XStringAnalyzer(TYPE_CONST, NULL, NULL, -.5),
                                OP_POWER),
                               OP_MINES);
                          break;
          case FUNC_ATG:  R = new XStringAnalyzer(TYPE_OPERATOR,
                               new XStringAnalyzer(TYPE_OPERATOR,
                                new XStringAnalyzer(TYPE_CONST, NULL, NULL, 1),
                                new XStringAnalyzer(TYPE_OPERATOR,
                                 new XStringAnalyzer(Left),
                                 new XStringAnalyzer(TYPE_CONST, NULL, NULL, 2),
                                 OP_POWER),
                                OP_PLUS),
                               new XStringAnalyzer(TYPE_CONST, NULL, NULL, -1),
                               OP_POWER);
                          break;
          case FUNC_LN:   R = new XStringAnalyzer(TYPE_OPERATOR,
                               new XStringAnalyzer(Left),
                               new XStringAnalyzer(TYPE_CONST, NULL, NULL, -1),
                              OP_POWER);
                          break;
          case FUNC_LG:   R = new XStringAnalyzer(TYPE_OPERATOR,
                                     new XStringAnalyzer(TYPE_OPERATOR,
                                         new XStringAnalyzer(Left),
                                         new XStringAnalyzer(TYPE_CONST, NULL, NULL, -1),
                                         OP_POWER),
                                     new XStringAnalyzer("ln(10)"),
                                     OP_DIVIDE);
                             break;
          case FUNC_EXP:  R = new XStringAnalyzer(this);
                          break;
          case FUNC_ABS:  throw XMathException("Exception: ABS presentes");
          default: throw XMathException("Exception: unknown function", 1);
        }
        D = new XStringAnalyzer(TYPE_OPERATOR, R, L, OP_MULTPLY);
        break;
   default: throw XMathException("Exception: unknown type", 1);
 }
 if(D) D->SSimplify();
return D;
}

// Deletes unnecessary parenthesis ex:"(expression)" and all spaces
XStringAnalyzer& XStringAnalyzer::DelParen(char* s){
int i, p, l;
char* buf;
  l=strlen(s);
  for(i=0, p=0; i<l; i++){                   //஢ઠ ୮ ᪮
     if(s[i]=='(')p++;
     if(s[i]==')')p--;
  }
  if(p)
    throw XMathException("Exception: illegal parenthesis", 1);

  buf = (char*)malloc((l+1)*sizeof(char));
  memset(buf, 0x0, (l+1)*sizeof(char));
  for(i=0, p=0; i<l; i++)                   //㤠 ஡
       if(s[i]!=' ')
          buf[p++] = s[i];
  strcpy(s, buf);

  l=strlen(s);
  while(l){                                  //㤠 譨 ᪮
    p=0;
    if(s[0]=='(' && s[l-1]==')')
      for(i=1, p=1; i<l-1; i++){
         if(s[i]=='(')p++;
         if(s[i]==')')p--;
         if(!p) break;
      }
    if(!p){
      free(buf);
      return *this;
    }
    strncpy(buf, &s[1], l-2);
    buf[l-2]=0;
    strcpy(s, buf);
    l=strlen(s);
  }
  free(buf);
  return *this;
}

// Main analyze: determines string(expression) type and constructs all
XStringAnalyzer& XStringAnalyzer::Analyze(char* Buf)
{
int i, p, l;
char *end;
  l=strlen(Buf);
  if(l==0){
      Type=TYPE_CONST;
      Param=0.;
      return *this;
  }
  if(!strcmpi(Buf, "x")){
    Type=TYPE_VARIABLE;
    return *this;
  }
  if(!strcmpi(Buf, "e")){
    Type=TYPE_CONST;
    Param=CONST_E;
    return *this;
  }
  if(!strcmpi(Buf, "pi")){
    Type=TYPE_CONST;
    Param=CONST_PI;
    return *this;
  }

  for(Param = OP_PLUS; Param<OP_POWER+1; Param++)
     for(i=l, p=0; i>-1;i--){
        if(Buf[i]=='(')p++;
        if(Buf[i]==')')p--;
        if(p==0)
        if(Buf[i]==OPERATORS[(int)Param]){
          Type = TYPE_OPERATOR;
          Buf[i] = 0x0;
          if((strlen(Buf)==0 && Param>OP_MINES) || strlen(&Buf[i+1])==0)
            throw XMathException("Exception: illegal syntax", 1);
          Left = new XStringAnalyzer(Buf);
          Right= new XStringAnalyzer(&Buf[i+1]);
          return *this;
        }
     }
  for(Param = FUNC_SIN; Param<FUNC_ABS+1; Param++){
       i = strlen(FUNCTIONS[(int)Param]);
       if(strnicmp(Buf, FUNCTIONS[(int)Param], i)==0 && Buf[i]=='(' && Buf[l-1]==')'){
         Type = TYPE_FUNCTION;
         if(strlen(&Buf[i])<3)                 // "(..)"
           throw XMathException("Exception: illegal syntax", 1);
         Left = new XStringAnalyzer(&Buf[i]);
         return *this;
       }
  }
  Param = strtod(Buf, &end);
  if(*end==NULL){
    Type=TYPE_CONST;
    return *this;
  }
  throw XMathException("Exception: illegal syntax", 1);
}

/*****************************************************************************/
//Simplifies expression
XStringAnalyzer& XStringAnalyzer::SSimplify()
{
if(!SIMPLIFY) return *this;
  SOrderOps();
  SCountConst();
  SCountPeer();
  SDeOrderOps();
return *this;
}

XStringAnalyzer& XStringAnalyzer::SOrderOps()
{
XStringAnalyzer* T;
  if(Type==TYPE_OPERATOR){
    Left->  SOrderOps();
    Right-> SOrderOps();
    switch((int)Param){
      case OP_MINES:
           Param=OP_PLUS;
           Right = new XStringAnalyzer(TYPE_OPERATOR,
                    new XStringAnalyzer(TYPE_CONST, NULL, NULL, -1),
                    Right, OP_MULTPLY);
           break;
      case OP_DIVIDE:
           Param=OP_MULTPLY;
           Right = new XStringAnalyzer(TYPE_OPERATOR,
                    Right, new XStringAnalyzer(TYPE_CONST, NULL, NULL, -1),
                    OP_POWER);
           break;
    }
  }
  if(Type==TYPE_FUNCTION)
     Left->SOrderOps();
return *this;
}


XStringAnalyzer& XStringAnalyzer::SDeOrderOps()
{
XStringAnalyzer* T;
  if(Type==TYPE_OPERATOR){
     Left->  SDeOrderOps();
     Right-> SDeOrderOps();
     switch((int)Param){
       case OP_MULTPLY:
            if(Left->Type==TYPE_CONST && Left->Param<0){
              Param=OP_MINES;
              Left->Param *= -1;
              if(Left->Param!=1)
              Right = new XStringAnalyzer(TYPE_OPERATOR,
                       Left, Right, OP_MULTPLY);
              Left = new XStringAnalyzer(TYPE_CONST, NULL, NULL, 0);
            }
            break;
       case OP_PLUS:
            if(Right->Type==TYPE_OPERATOR && (int)Right->Param==OP_MINES &&
               Right->Left->Type==TYPE_CONST && Right->Left->Param==0 ){
               Param=OP_MINES;
               T = new XStringAnalyzer(Right->Right);
               delete Right;
               Right = T;
            }
            break;
     }
  }
  if(Type==TYPE_FUNCTION)
     Left->SDeOrderOps();
return *this;
}

//Counts all possible constants
XStringAnalyzer& XStringAnalyzer::SCountConst(){
XStringAnalyzer *T;
double Val;
  if(Type==TYPE_OPERATOR){
     Left-> SCountConst();
     Right->SCountConst();
     if(!Left->isConst() && !Right->isConst()) return *this;
     if(Left->isConst() && Right->isConst()){
         try { Val = Count(0);
         } catch(XMathException e) { return *this; }
         delete Left;
         delete Right;
         Type = TYPE_CONST;
         Param = Val;
         return *this;
     }
     if(Left->isConst()){
       try { Val = Left->Count(0);
       } catch(XMathException e) { return *this; }
       if(Val!=0 && Val!=1) return *this;
       if(Val==0){
         if((int)Param==OP_PLUS){
           delete Left;
           T = Right;
           Clone(Right);
           delete T;
         } else {
           Type=TYPE_CONST;
           Param=0;
           delete Left;
           delete Right;
         }
         return *this;
       }
       if(Val==1){
         if(Param==OP_MULTPLY){
           delete Left;
           T = Right;
           Clone(Right);
           delete T;
         } else if((int)Param==OP_POWER){
           Type=TYPE_CONST;
           Param=1;
           delete Left;
           delete Right;
         }
         return *this;
       }
       return *this;
     }
     if(Right->isConst()){
       try { Val = Right->Count(0);
       } catch(XMathException e) { return *this; }
       if(Val!=0 && Val!=1) return *this;
       if(Val==0){
         if(Param==OP_PLUS){
           delete Right;
           T = Left;
           Clone(Left);
           delete T;
         } else {
           Type=TYPE_CONST;
           Param=((int)Param==OP_MULTPLY)?0:1;
           delete Left;
           delete Right;
         }
         return *this;
       }
       if(Val==1){
         if((int)Param!=OP_PLUS){
           delete Right;
           T = Left;
           Clone(Left);
           delete T;
         }
       }
       return *this;
     }
  }
  if(Type==TYPE_FUNCTION){
     Left->SCountConst();
     if(Left->isConst()){
       try { Val = Count(0);
       } catch(XMathException e) { return *this; }
       delete Left;
       Type = TYPE_CONST;
       Param = Val;
       return *this;
     }
  }
return *this;
}


XStringAnalyzer& XStringAnalyzer::SCountPeer()
{
XStringAnalyzer* Temp, *Coef, **I2, **J2;
XStringAnalyzer** Operand=NULL, **Coeffs=NULL;
double Co;
int size=0, i, j, k, flag, F=0;
  if(Type==TYPE_OPERATOR){
    Left->SCountPeer();
    Right->SCountPeer();
    if((int)Param==OP_POWER){
      if(Left->Type==TYPE_OPERATOR && Left->Param==Param){
         Temp  = new XStringAnalyzer(Left->Left);
         Right = new XStringAnalyzer(TYPE_OPERATOR, new XStringAnalyzer(Left->Right), Right, OP_MULTPLY);
         delete Left;
         Left = Temp;
         SCountConst();
         Right->SCountPeer();
       }
       return *this;
    }
    Operand=FindOperands(Operand, (int)Param, &size);
    Coeffs = (XStringAnalyzer**)malloc(sizeof(XStringAnalyzer*)*size);
    delete Left; delete Right;

    for(i=0,Co=((int)Param==OP_PLUS)?0:1;i<size;i++)
       if(Operand[i]->Type==TYPE_CONST){
         ((int)Param==OP_PLUS)?Co+=Operand[i]->Param:Co*=Operand[i]->Param;
         delete Operand[i];
         Operand[i]=NULL;
       }
    for(i=0, j=i;i<size;i++)
       if(Operand[i])
         Operand[j++]=Operand[i];
    size = j;

    Coef = new XStringAnalyzer(TYPE_CONST, NULL, NULL, Co);
    for(i=0; i<size; i++)
       if(!(Operand[i]->Type==TYPE_OPERATOR && Operand[i]->Param==Param+2))
         Operand[i] = new XStringAnalyzer(TYPE_OPERATOR, Operand[i],
                       new XStringAnalyzer(TYPE_CONST, NULL, NULL, 1), Param+2);
    for(i=0; i<size-1; i++){
       for(j=i+1; j<size; j++){
          flag=1;
          if((int)Param==OP_PLUS){
            if(Operand[j]->Left->isEqual(Operand[i]->Left) && !(Operand[j]->Left->Type==TYPE_CONST && Operand[j]->Left->Param==1))
              { I2=&Operand[i]->Right; J2=&Operand[j]->Right;}
            else if(Operand[j]->Right->isEqual(Operand[i]->Left) && !(Operand[j]->Right->Type==TYPE_CONST && Operand[j]->Right->Param==1))
              { I2=&Operand[i]->Right; J2=&Operand[j]->Left;}
            else if(Operand[j]->Left->isEqual(Operand[i]->Right) && !(Operand[j]->Left->Type==TYPE_CONST && Operand[j]->Left->Param==1))
              { I2=&Operand[i]->Left;  J2=&Operand[j]->Right;}
            else if(Operand[j]->Right->isEqual(Operand[i]->Right) && !(Operand[j]->Right->Type==TYPE_CONST && Operand[j]->Right->Param==1))
              { I2=&Operand[i]->Left;  J2=&Operand[j]->Left;}
            else flag=0;
           } else {
             if(Operand[j]->Left->isEqual(Operand[i]->Left))       { I2=&Operand[i]->Right; J2=&Operand[j]->Right;}
             else flag=0;
           }
           if(flag){
             *I2 = new XStringAnalyzer(TYPE_OPERATOR,
                    *I2, new XStringAnalyzer(*J2), OP_PLUS);
             delete Operand[j];
             Operand[j]=NULL; F=1;
           }
       }
       for(j=i+1, k=j;j<size;j++)
          if(Operand[j])
            Operand[k++]=Operand[j];
       size=k;
    }
    if(size){
      Temp = Operand[0];
      for(i=1; i<size; i++)
         Temp = new XStringAnalyzer(TYPE_OPERATOR, Temp, Operand[i], Param);
      Left=Coef;
      Right=Temp;
      SCountConst();
      if(F) Right->SCountPeer();
      free(Operand);
      free(Coeffs);
      SCountConst();
      return *this;
    } else {
      Clone(Coef);
      free(Operand);
      free(Coeffs);
      return *this;
    }
  }
  if(Type==TYPE_FUNCTION){
    Left->SCountPeer();
    return *this;
  }
return *this;
}

/*****************************************************************************/
// Determine if function depends on variable "x"
int XStringAnalyzer::isConst()
{
int rc=1;
  if(Type==TYPE_OPERATOR)
     if(!(Left->isConst() && Right->isConst())) rc=0;
  if(Type==TYPE_FUNCTION)
     if(!Left->isConst()) rc=0;
  if(Type==TYPE_VARIABLE) rc=0;
return rc;
}

// Determine if two functions are identical
int XStringAnalyzer::isEqual(XStringAnalyzer* s)
{
  if(Type==s->Type && Param==s->Param)
     switch(Type){
       case TYPE_VARIABLE:
       case TYPE_CONST:
          return 1;
       case TYPE_OPERATOR:
          return (Left->isEqual(s->Left) && Right->isEqual(s->Right));
       case TYPE_FUNCTION:
          return Left->isEqual(s->Left);
     }
  return 0;
}
/*****************************************************************************/

XStringAnalyzer** XStringAnalyzer::FindOperands(XStringAnalyzer** Buf, int Oper, int* size)
{
  if(Type==TYPE_OPERATOR && (int)Param==Oper)
    {
      Buf=Left->FindOperands(Buf, Oper, size);
      Buf=Right->FindOperands(Buf, Oper, size);
    }
  else
    {
      Buf=(XStringAnalyzer**)realloc(Buf, ++(*size)*sizeof(XStringAnalyzer*));
      Buf[*size-1]=new XStringAnalyzer(this);
    }
return Buf;
}
